@@ -1,5 +1,5 @@
 /* command.h -- Shared command execution for k4start and k5start. */
-/* $Id: command.h 2588 2006-06-13 20:54:44Z rra $ */
+/* $Id: command.h 3009 2007-03-04 04:42:26Z rra $ */
 
 #ifndef COMMAND_H
 #define COMMAND_H 1
@@ -8,7 +8,7 @@
 
 /* Run the given aklog command.  If verbose is true, print some more output to
    standard output about the exit status. */
-int run_aklog(const char *aklog, int verbose);
+void run_aklog(const char *aklog, int verbose);
 
 /* Start a command, executing the given command with the given argument vector
    (which includes argv[0]).  Returns the PID or -1 on error.  This function
