@@ -1,9 +1,9 @@
-/*  $Id: k4start.c 2597 2006-06-24 02:39:40Z rra $
+/*  $Id: k4start.c 3009 2007-03-04 04:42:26Z rra $
 **
 **  Kerberos v4 kinit replacement suitable for daemon authentication.
 **
 **  Copyright 1987, 1988 by the Massachusetts Institute of Technology.
-**  Copyright 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2004, 2005
+**  Copyright 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2004, 2005, 2007
 **      Board of Trustees, Leland Stanford Jr. University
 **
 **  For copying and distribution information, please see README.
@@ -17,74 +17,23 @@
 **  It is based very heavily on a modified Kerberos v4 kinit.
 */
 
-#include "config.h"
-#include "command.h"
+#include <config.h>
+#include <system.h>
+#include <portable/kafs.h>
+#include <portable/krb4.h>
+#include <portable/time.h>
 
 #include <errno.h>
 #include <pwd.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#if HAVE_K_SETPAG
-# if HAVE_KAFS_H
-#  include <kafs.h>
-# endif
-#elif HAVE_LSETPAG
-int lsetpag(void);
-# define k_hasafs() (1)
-# define k_setpag() lsetpag()
-#else
-# define k_hasafs() (1)
-# define k_setpag() (0)
-#endif
-
-#ifdef HAVE_KERBEROSIV_KRB_H
-# include <kerberosIV/krb.h>
-#else
-# include <krb.h>
-#endif
-
-#ifndef HAVE_DAEMON
-extern int daemon(int, int);
-#endif
-
-#ifndef HAVE_MKSTEMP
-extern int mkstemp(char *);
-#endif
-
-/* We default to a ten hour ticket lifetime if the Kerberos headers don't
-   provide a value. */
-#ifndef DEFAULT_TKT_LIFE
-# define DEFAULT_TKT_LIFE 120
-#endif
+#include <command.h>
 
 /* The number of seconds of fudge to add to the check for whether we need to
    obtain a new ticket.  This is here to make sure that we don't wake up just
    as the ticket is expiring. */
 #define EXPIRE_FUDGE 120
 
-/* Make sure everything compiles even if no aklog program was found by
-   configure. */
-#ifndef PATH_AKLOG
-# define PATH_AKLOG ""
-#endif
-
 /* Holds the various command-line options for passing to functions. */
 struct options {
     char aname[ANAME_SZ];
@@ -182,13 +131,17 @@ ticket_expired(struct options *options)
 {
     CREDENTIALS cr;
     int status;
-    time_t now, then;
+    time_t now, then, offset;
 
     status = krb_get_cred(options->sname, options->sinst, options->realm, &cr);
     if (status == KSUCCESS) {
         now = time(NULL);
         then = krb_life_to_time(cr.issue_date, cr.lifetime);
-        if (then < now + 60 * options->keep_ticket + EXPIRE_FUDGE)
+        if (options->happy_ticket > 0)
+            offset = 60 * options->happy_ticket;
+        else
+            offset = 60 * options->keep_ticket + EXPIRE_FUDGE;
+        if (then < now + offset)
             status = RD_AP_EXP;
     }
     return status;
@@ -196,16 +149,12 @@ ticket_expired(struct options *options)
 
 
 /*
-**  Authenticate, given a set of options.  Also handles running aklog if
-**  requested.  Normally dies on failure, but if authentication succeeds and
-**  aklog just failed, return the exit status of aklog instead (or 7 if it
-**  couldn't be run).
+**  Authenticate, given a set of options, and dies on failure.
 */
-static int
-authenticate(struct options *options, const char *aklog)
+static void
+authenticate(struct options *options)
 {
     int k_errno;
-    int status = 0;
 
     if (options->verbose) {
         printf("Principal: %s.%s@%s\n", options->aname, options->inst,
@@ -244,11 +193,6 @@ authenticate(struct options *options, const char *aklog)
         else
             die("Kerberos error: %s", krb_err_txt[k_errno]);
     }
-
-    /* If requested, run the aklog program. */
-    if (options->run_aklog && !options->no_aklog)
-        status = run_aklog(aklog, options->verbose);
-    return status;
 }
 
 
@@ -258,7 +202,7 @@ main(int argc, char *argv[])
     struct options options;
     int k_errno, opt, result;
     char *username = NULL;
-    char *aklog = NULL;
+    const char *aklog = NULL;
     char **command = NULL;
     char *pidfile = NULL;
     int background = 0;
@@ -365,6 +309,8 @@ main(int argc, char *argv[])
     if (lifetime > 0 && options.keep_ticket > lifetime)
         die("-K limit %d must be smaller than lifetime %d",
             options.keep_ticket, options.lifetime);
+    if (options.happy_ticket > 0 && options.keep_ticket > 0)
+        die("-H and -K options cannot be used at the same time");
 
     /* Check to see if KINIT_PROG is set.  If it is, and no_aklog is not set,
        set run_aklog, since setting that environment variable changes the
@@ -461,7 +407,7 @@ main(int argc, char *argv[])
     }
 
     /* If we're just checking the service ticket, do that and exit if okay. */
-    if (options.happy_ticket > 0)
+    if (options.happy_ticket > 0 && command == NULL)
         if (!ticket_expired(&options))
             exit(0);
 
@@ -476,8 +422,15 @@ main(int argc, char *argv[])
         }
     }
 
-    /* Now, do the actual authentication. */
-    status = authenticate(&options, aklog);
+    /* Now, the actual authentication part.  If -H wasn't set, always
+       authenticate.  If -H was set, authenticate only if the ticket isn't
+       expired. */
+    if (options.happy_ticket == 0 || ticket_expired(&options))
+        authenticate(&options);
+
+    /* If requested, run the aklog program. */
+    if (options.run_aklog && !options.no_aklog)
+        run_aklog(aklog, options.verbose);
 
     /* If told to background, background ourselves.  We do this late so that
        we can report initial errors.  We have to do this before spawning the
@@ -521,17 +474,19 @@ main(int argc, char *argv[])
                     die("waitpid for %lu failed: %s", (unsigned long) child,
                         strerror(errno));
                 if (result > 0)
-                    goto done;
+                    break;
             }
             timeout.tv_sec = options.keep_ticket * 60;
             timeout.tv_usec = 0;
             select(0, NULL, NULL, NULL, &timeout);
-            if (ticket_expired(&options))
-                status = authenticate(&options, aklog);
+            if (ticket_expired(&options)) {
+                authenticate(&options);
+                if (options.run_aklog && !options.no_aklog)
+                    run_aklog(aklog, options.verbose);
+            }
         }
     }
 
-done:
     /* Otherwise, or when we're done, exit.  clean_cache is only set if we
        used mkstemp to generate the ticket cache name. */
     if (clean_cache)
