@@ -1,9 +1,9 @@
-/*  $Id: k5start.c 2597 2006-06-24 02:39:40Z rra $
+/*  $Id: k5start.c 3009 2007-03-04 04:42:26Z rra $
 **
 **  Kerberos v5 kinit replacement suitable for daemon authentication.
 **
 **  Copyright 1987, 1988 by the Massachusetts Institute of Technology.
-**  Copyright 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2004, 2005, 2006
+**  Copyright 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007
 **      Board of Trustees, Leland Stanford Jr. University
 **
 **  For copying and distribution information, please see README.
@@ -19,71 +19,18 @@
 **  supports more useful options, but -K and -H are still unique to it.
 */
 
-#include "config.h"
-#include "command.h"
+#include <config.h>
+#include <system.h>
+#include <portable/kafs.h>
+#include <portable/krb5.h>
+#include <portable/time.h>
 
 #include <errno.h>
 #include <netdb.h>
 #include <pwd.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#include <krb5.h>
-
-#if HAVE_K_SETPAG
-# if HAVE_KAFS_H
-#  include <kafs.h>
-# endif
-#elif HAVE_LSETPAG
-int lsetpag(void);
-# define k_hasafs() (1)
-# define k_setpag() lsetpag()
-#else
-# define k_hasafs() (1)
-# define k_setpag() (0)
-#endif
-
-/* __attribute__ is available in gcc 2.5 and later, but only with gcc 2.7
-   could you use the __format__ form of the attributes, which is what we use
-   (to avoid confusion with other macros). */
-#ifndef __attribute__
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __attribute__(spec)   /* empty */
-# endif
-#endif
-
-#ifndef HAVE_DAEMON
-extern int daemon(int, int);
-#endif
-
-#ifndef HAVE_KRB5_ERR
-extern krb5_error_code krb5_err(krb5_context, int, krb5_error_code,
-                                const char *, ...)
-    __attribute__((__format__(printf, 4, 5)));
-extern krb5_error_code krb5_warn(krb5_context, krb5_error_code,
-                                 const char *, ...)
-    __attribute__((__format__(printf, 3, 4)));
-#endif
-
-#ifndef HAVE_MKSTEMP
-extern int mkstemp(char *);
-#endif
+#include <command.h>
 
 /* The default ticket lifetime in minutes.  Default to 10 hours. */
 #define DEFAULT_LIFETIME (10 * 60)
@@ -93,12 +40,6 @@ extern int mkstemp(char *);
    as the ticket is expiring. */
 #define EXPIRE_FUDGE 120
 
-/* Make sure everything compiles even if no aklog program was found by
-   configure. */
-#ifndef PATH_AKLOG
-# define PATH_AKLOG ""
-#endif
-
 /* Holds the various command-line options for passing to functions, after
    processing in the main routine and conversion to internal K5 data
    structures where appropriate. */
@@ -189,7 +130,7 @@ usage(int status)
 **  MIT Kerberos and Heimdal, unfortunately.
 */
 static char *
-get_realm(krb5_context ctx, krb5_principal princ)
+get_realm(krb5_context ctx UNUSED, krb5_principal princ)
 {
 #ifdef HAVE_KRB5_REALM
     krb5_realm *realm;
@@ -217,7 +158,7 @@ static krb5_error_code
 ticket_expired(krb5_context ctx, struct options *options)
 {
     krb5_creds increds, *outcreds = NULL;
-    time_t now, then;
+    time_t now, then, offset;
     int status;
 
     /* Obtain the ticket. */
@@ -227,11 +168,18 @@ ticket_expired(krb5_context ctx, struct options *options)
     status = krb5_get_credentials(ctx, 0, options->ccache, &increds,
                                   &outcreds);
 
-    /* Check the expiration time. */
+    /* Check the expiration time.  We may be looking for a ticket that lasts a
+       particuliar length of time based on either keep_ticket or
+       happy_ticket.  Only one of those options will be set; at least one of
+       them will always be zero. */
     if (status == 0) {
         now = time(NULL);
         then = outcreds->times.endtime;
-        if (then < now + 60 * options->keep_ticket + EXPIRE_FUDGE)
+        if (options->happy_ticket > 0)
+            offset = 60 * options->happy_ticket;
+        else
+            offset = 60 * options->keep_ticket + EXPIRE_FUDGE;
+        if (then < now + offset)
             status = KRB5KRB_AP_ERR_TKT_EXPIRED;
     }
 
@@ -245,15 +193,12 @@ ticket_expired(krb5_context ctx, struct options *options)
 
 /*
 **  Authenticate, given the context and the processed command-line options.
-**  Also takes care of running aklog if requested.  Normally dies on failure,
-**  but if authentication succeeds and aklog just failed, return the exit
-**  status of aklog instead (or 7 if it couldn't be run).
+**  Dies on failure.
 */
-static int
+static void
 authenticate(krb5_context ctx, struct options *options)
 {
     int k5_errno;
-    int status = 0;
     krb5_keytab k5_keytab = NULL;
     krb5_creds creds;
 
@@ -316,14 +261,6 @@ authenticate(krb5_context ctx, struct options *options)
     krb5_free_cred_contents(ctx, &creds);
     if (k5_keytab != NULL)
         krb5_kt_close(ctx, k5_keytab);
-
-    /* If requested, run the aklog program.  IRIX 6.5's WEXITSTATUS() macro is
-       broken and can't cope with being called directly on the return value of
-       system().  If we can't execute the aklog program, set the exit status
-       to an arbitrary but distinct value. */
-    if (options->run_aklog)
-        status = run_aklog(options->aklog, options->verbose);
-    return status;
 }
 
 
@@ -478,7 +415,9 @@ main(int argc, char *argv[])
     if (principal != NULL && strchr(principal, '/') != NULL && inst != NULL)
         die("instance specified in the principal and with -i");
     if (search_keytab && options.keytab == NULL)
-        die("-p option requires a keytab be specified with -f");
+        die("-U option requires a keytab be specified with -f");
+    if (options.happy_ticket > 0 && options.keep_ticket > 0)
+        die("-H and -K options cannot be used at the same time");
 
     /* Set aklog from KINIT_PROG or the compiled-in default. */
     options.aklog = getenv("KINIT_PROG");
@@ -615,7 +554,7 @@ main(int argc, char *argv[])
     krb5_get_init_creds_opt_set_tkt_life(&options.kopts, life_secs);
 
     /* If we're just checking the service ticket, do that and exit if okay. */
-    if (options.happy_ticket > 0)
+    if (options.happy_ticket > 0 && command == NULL)
         if (!ticket_expired(ctx, &options))
             exit(0);
 
@@ -630,8 +569,15 @@ main(int argc, char *argv[])
         }
     }
 
-    /* Now, the actual authentication part. */
-    status = authenticate(ctx, &options);
+    /* Now, the actual authentication part.  If -H wasn't set, always
+       authenticate.  If -H was set, authenticate only if the ticket isn't
+       expired. */
+    if (options.happy_ticket == 0 || ticket_expired(ctx, &options))
+        authenticate(ctx, &options);
+
+    /* If requested, run the aklog program. */
+    if (options.run_aklog)
+        run_aklog(options.aklog, options.verbose);
 
     /* If told to background, background ourselves.  We do this late so that
        we can report initial errors.  We have to do this before spawning the
@@ -657,7 +603,7 @@ main(int argc, char *argv[])
        X our ticket cache isn't going to survive the setsid that daemon does
        and we've now lost our credentials. */
     if (background && ticket_expired(ctx, &options))
-        status = authenticate(ctx, &options);
+        authenticate(ctx, &options);
 
     /* Spawn the external command, if we were told to run one. */
     if (command != NULL) {
@@ -682,17 +628,19 @@ main(int argc, char *argv[])
                     die("waitpid for %lu failed: %s", (unsigned long) child,
                         strerror(errno));
                 if (result > 0)
-                    goto done;
+                    break;
             }
             timeout.tv_sec = options.keep_ticket * 60;
             timeout.tv_usec = 0;
             select(0, NULL, NULL, NULL, &timeout);
-            if (ticket_expired(ctx, &options))
-                status = authenticate(ctx, &options);
+            if (ticket_expired(ctx, &options)) {
+                authenticate(ctx, &options);
+                if (options.run_aklog)
+                    run_aklog(options.aklog, options.verbose);
+            }
         }
     }
 
-done:
     /* Otherwise, or when we're done, exit.  clean_cache is only set if we
        used mkstemp to generate the ticket cache name. */
     if (clean_cache)
