@@ -1,8 +1,8 @@
-/*  $Id: krenew.c 2597 2006-06-24 02:39:40Z rra $
+/*  $Id: krenew.c 3009 2007-03-04 04:42:26Z rra $
 **
 **  Automatically renew a Kerberos v5 ticket.
 **
-**  Copyright 2006 Board of Trustees, Leland Stanford Jr. University
+**  Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
 **
 **  For copying and distribution information, please see README.
 **
@@ -13,76 +13,21 @@
 **  renewed any longer.
 */
 
-#include "config.h"
-#include "command.h"
+#include <config.h>
+#include <system.h>
+#include <portable/kafs.h>
+#include <portable/krb5.h>
+#include <portable/time.h>
 
 #include <errno.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <unistd.h>
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#include <krb5.h>
-
-#if HAVE_K_SETPAG
-# if HAVE_KAFS_H
-#  include <kafs.h>
-# endif
-#elif HAVE_LSETPAG
-int lsetpag(void);
-# define k_hasafs() (1)
-# define k_setpag() lsetpag()
-#else
-# define k_hasafs() (1)
-# define k_setpag() (0)
-#endif
-
-/* __attribute__ is available in gcc 2.5 and later, but only with gcc 2.7
-   could you use the __format__ form of the attributes, which is what we use
-   (to avoid confusion with other macros). */
-#ifndef __attribute__
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __attribute__(spec)   /* empty */
-# endif
-#endif
-
-#ifndef HAVE_DAEMON
-extern int daemon(int, int);
-#endif
-
-#ifndef HAVE_KRB5_ERR
-extern krb5_error_code krb5_err(krb5_context, int, krb5_error_code,
-                                const char *, ...)
-    __attribute__((__format__(printf, 4, 5)));
-extern krb5_error_code krb5_warn(krb5_context, krb5_error_code,
-                                 const char *, ...)
-    __attribute__((__format__(printf, 3, 4)));
-#endif
+#include <command.h>
 
 /* The number of seconds of fudge to add to the check for whether we need to
    obtain a new ticket.  This is here to make sure that we don't wake up just
    as the ticket is expiring. */
 #define EXPIRE_FUDGE (2 * 60)
 
-/* Make sure everything compiles even if no aklog program was found by
-   configure. */
-#ifndef PATH_AKLOG
-# define PATH_AKLOG ""
-#endif
-
 /* The usage message. */
 const char usage_message[] = "\
 Usage: krenew [options] [command]\n\
@@ -138,7 +83,7 @@ usage(int status)
 **  MIT Kerberos and Heimdal, unfortunately.
 */
 static char *
-get_realm(krb5_context ctx, krb5_principal princ)
+get_realm(krb5_context ctx UNUSED, krb5_principal princ)
 {
 #ifdef HAVE_KRB5_REALM
     krb5_realm *realm;
@@ -218,11 +163,9 @@ ticket_expired(krb5_context ctx, krb5_ccache cache, int keep_ticket)
 
 /*
 **  Renew the user's tickets, exiting with an error if this isn't possible.
-**  Normally dies on failure, but if authentication succeeds and aklog just
-**  failed, return the exit status of aklog instead.
 */
-static int
-renew(krb5_context ctx, krb5_ccache cache, const char *aklog, int verbose)
+static void
+renew(krb5_context ctx, krb5_ccache cache, int verbose)
 {
     int status;
     krb5_principal user;
@@ -285,12 +228,6 @@ renew(krb5_context ctx, krb5_ccache cache, const char *aklog, int verbose)
     if (out != NULL)
         krb5_free_creds(ctx, out);
 #endif
-
-    /* If requested, run the aklog program. */
-    if (aklog != NULL && aklog[0] != '\0')
-        status = run_aklog(aklog, verbose);
-
-    return status;
 }
 
 
@@ -303,9 +240,9 @@ main(int argc, char *argv[])
     char *pidfile = NULL;
     int background = 0;
     int keep_ticket = 0;
-    int run_aklog = 0;
+    int do_aklog = 0;
     int verbose = 0;
-    char *aklog = NULL;
+    const char *aklog = NULL;
     krb5_context ctx;
     krb5_ccache cache;
     int status = 0;
@@ -317,7 +254,7 @@ main(int argc, char *argv[])
         case 'b': background = 1;               break;
         case 'h': usage(0);                     break;
         case 'p': pidfile = optarg;             break;
-        case 't': run_aklog = 1;                break;
+        case 't': do_aklog = 1;                 break;
         case 'v': verbose = 1;                  break;
 
         case 'K':
@@ -351,7 +288,7 @@ main(int argc, char *argv[])
     aklog = getenv("KINIT_PROG");
     if (aklog == NULL)
         aklog = PATH_AKLOG;
-    if (aklog[0] == '\0' && run_aklog)
+    if (aklog[0] == '\0' && do_aklog)
         die("set KINIT_PROG to specify the path to aklog");
 
     /* Establish a K5 context and set the ticket cache. */
@@ -375,7 +312,7 @@ main(int argc, char *argv[])
 
     /* If built with setpag support and we're running a command, create the
        new PAG now before the first authentication. */
-    if (command != NULL && run_aklog) {
+    if (command != NULL && do_aklog) {
         if (k_hasafs()) {
             if (k_setpag() < 0)
                 die("unable to create PAG: %s", strerror(errno));
@@ -386,7 +323,11 @@ main(int argc, char *argv[])
 
     /* Now, do the initial ticket renewal even if it's not necessary so that
        we can catch any problems. */
-    status = renew(ctx, cache, run_aklog ? aklog : NULL, verbose);
+    renew(ctx, cache, verbose);
+
+    /* If requested, run the aklog program. */
+    if (do_aklog)
+        run_aklog(aklog, verbose);
 
     /* If told to background, background ourselves.  We do this late so that
        we can report initial errors.  We have to do this before spawning the
@@ -427,16 +368,19 @@ main(int argc, char *argv[])
                     die("waitpid for %lu failed: %s", (unsigned long) child,
                         strerror(errno));
                 if (result > 0)
-                    goto done;
+                    break;
             }
             timeout.tv_sec = keep_ticket * 60;
             timeout.tv_usec = 0;
             select(0, NULL, NULL, NULL, &timeout);
-            if (ticket_expired(ctx, cache, keep_ticket))
-                status = renew(ctx, cache, run_aklog ? aklog : NULL, verbose);
+            if (ticket_expired(ctx, cache, keep_ticket)) {
+                renew(ctx, cache, verbose);
+                if (do_aklog)
+                    run_aklog(aklog, verbose);
+            }
         }
     }
 
-done:
+    /* All done. */
     exit(status);
 }
